"""Diag plugin.

Separate from DHCPD until we definitively understand the different elements.

"""
# Copyright © 2022 Appropriate Solutions, Inc. All rights reserved.

from .diag import Plugin  # NOQA
